"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _has2 = _interopRequireDefault(require("lodash/has"));

var _get2 = _interopRequireDefault(require("lodash/get"));

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _reactAce = _interopRequireDefault(require("react-ace"));

var _patchEvent = require("part:@sanity/form-builder/patch-event");

var _CodeInput = _interopRequireDefault(require("./CodeInput.css"));

var _default = _interopRequireDefault(require("part:@sanity/components/formfields/default"));

var _default2 = _interopRequireDefault(require("part:@sanity/components/fieldsets/default"));

var _default3 = _interopRequireDefault(require("part:@sanity/components/selects/default"));

var _createHighlightMarkers = _interopRequireDefault(require("./createHighlightMarkers"));

var _config = require("./config");

require("brace/mode/batchfile");

require("brace/mode/css");

require("brace/mode/html");

require("brace/mode/javascript");

require("brace/mode/json");

require("brace/mode/jsx");

require("brace/mode/markdown");

require("brace/mode/php");

require("brace/mode/sh");

require("brace/mode/text");

require("brace/theme/github");

require("brace/theme/monokai");

require("brace/theme/terminal");

require("brace/theme/tomorrow");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/* eslint-enable import/no-unassigned-import */
function compareNumbers(numA, numB) {
  return numA - numB;
}

class CodeInput extends _react.PureComponent {
  constructor() {
    super(...arguments);

    _defineProperty(this, "handleCodeChange", code => {
      const _this$props = this.props,
            type = _this$props.type,
            onChange = _this$props.onChange;
      const path = ['code'];
      const fixedLanguage = (0, _get2.default)(type, 'options.language');
      onChange(_patchEvent.PatchEvent.from([(0, _patchEvent.setIfMissing)({
        _type: type.name,
        language: fixedLanguage
      }), code ? (0, _patchEvent.set)(code, path) : (0, _patchEvent.unset)(path)]));
    });

    _defineProperty(this, "handleToggleSelectLine", lineNumber => {
      const _this$props2 = this.props,
            type = _this$props2.type,
            onChange = _this$props2.onChange,
            value = _this$props2.value;
      const path = ['highlightedLines'];
      const highlightedLines = value && value.highlightedLines || [];
      let position = highlightedLines.indexOf(lineNumber);
      const patches = [(0, _patchEvent.setIfMissing)({
        _type: type.name
      }), (0, _patchEvent.setIfMissing)([], ['highlightedLines'])];
      const addLine = position === -1;

      if (addLine) {
        // New element, figure out where to add it so it sorts correctly
        const sorted = highlightedLines.concat(lineNumber).sort(compareNumbers);
        position = sorted.indexOf(lineNumber);
        patches.push((0, _patchEvent.insert)([lineNumber], 'before', path.concat(position === sorted.length - 1 ? -1 : position)));
      } else if (highlightedLines.length === 1) {
        // Last element removed, unset whole path
        patches.push((0, _patchEvent.unset)(path)); // Temporary workaround for bug in react-ace
        // (https://github.com/securingsincity/react-ace/issues/229)

        const editor = this.editor // Remove all markers from editor
        ;
        [true, false].forEach(inFront => {
          const currentMarkers = editor.getSession().getMarkers(inFront);
          Object.keys(currentMarkers).forEach(marker => {
            editor.getSession().removeMarker(currentMarkers[marker].id);
          });
        });
      } else {
        // Removed, but not the last element, remove single item
        patches.push((0, _patchEvent.unset)(path.concat(position)));
      }

      onChange(_patchEvent.PatchEvent.from(patches));
    });

    _defineProperty(this, "handleGutterMouseDown", event => {
      const target = event.domEvent.target;

      if (target.classList.contains('ace_gutter-cell')) {
        const row = event.getDocumentPosition().row;
        this.handleToggleSelectLine(row + 1); // Ace starts at row 0
      }
    });

    _defineProperty(this, "handleEditorLoad", editor => {
      this.editor = editor;
      this.editor.focus();
      this.editor.on('guttermousedown', this.handleGutterMouseDown);
    });

    _defineProperty(this, "handleLanguageChange", item => {
      const _this$props3 = this.props,
            type = _this$props3.type,
            onChange = _this$props3.onChange;
      const path = ['language'];
      onChange(_patchEvent.PatchEvent.from([(0, _patchEvent.setIfMissing)({
        _type: type.name
      }), item ? (0, _patchEvent.set)(item.value, path) : (0, _patchEvent.unset)(path)]));
    });

    _defineProperty(this, "renderEditor", () => {
      const _this$props4 = this.props,
            value = _this$props4.value,
            type = _this$props4.type;
      const fixedLanguage = (0, _get2.default)(type, 'options.language');
      return _react.default.createElement(_reactAce.default, {
        className: _CodeInput.default.aceEditor,
        mode: value && value.language || fixedLanguage || 'text',
        theme: this.getTheme(),
        width: "100%",
        onChange: this.handleCodeChange,
        name: `${this._inputId}__aceEditor`,
        value: value && value.code || '',
        markers: value && value.highlightedLines ? (0, _createHighlightMarkers.default)(value.highlightedLines) : null,
        onLoad: this.handleEditorLoad,
        tabSize: 2,
        setOptions: _config.ACE_SET_OPTIONS,
        editorProps: _config.ACE_EDITOR_PROPS
      });
    });
  }

  focus() {
    this.editor.focus();
  }

  componentWillUnmount() {
    this.editor.removeListener('guttermousedown', this.handleGutterMouseDown);
  }

  getLanguageAlternatives() {
    return (0, _get2.default)(this.props.type, 'options.languageAlternatives') || _config.SUPPORTED_LANGUAGES;
  }

  getTheme() {
    const preferredTheme = (0, _get2.default)(this.props.type, 'options.theme');
    return preferredTheme && _config.SUPPORTED_THEMES.find(theme => theme === preferredTheme) ? preferredTheme : _config.DEFAULT_THEME;
  }

  render() {
    const _this$props5 = this.props,
          value = _this$props5.value,
          type = _this$props5.type,
          level = _this$props5.level;
    const languages = this.getLanguageAlternatives().slice();

    if ((0, _has2.default)(type, 'options.language')) {
      return _react.default.createElement(_default2.default, {
        legend: type.title,
        description: type.description,
        level: level
      }, this.renderEditor());
    }

    const selectedLanguage = value && value.language ? this.getLanguageAlternatives().find(item => item.value === value.language) : undefined;

    if (!selectedLanguage) {
      languages.unshift({
        title: 'Select language'
      });
    }

    const languageField = type.fields.find(field => field.name === 'language');
    return _react.default.createElement(_default2.default, {
      legend: type.title,
      description: type.description,
      level: level
    }, _react.default.createElement(_default.default, {
      level: level + 1,
      label: languageField.type.title
    }, _react.default.createElement(_default3.default, {
      onChange: this.handleLanguageChange,
      value: selectedLanguage,
      items: languages
    })), _react.default.createElement(_default.default, {
      label: selectedLanguage && selectedLanguage.title || 'Code',
      level: level + 1
    }, this.renderEditor()));
  }

}

exports.default = CodeInput;

_defineProperty(CodeInput, "propTypes", {
  level: _propTypes.default.number.isRequired,
  value: _propTypes.default.shape({
    _type: _propTypes.default.string,
    code: _propTypes.default.string,
    language: _propTypes.default.string,
    highlightedLines: _propTypes.default.array
  }),
  type: _propTypes.default.shape({
    name: _propTypes.default.string,
    title: _propTypes.default.string,
    description: _propTypes.default.string,
    fields: _propTypes.default.arrayOf(_propTypes.default.shape({
      name: _propTypes.default.string.isRequired
    }))
  }).isRequired,
  onChange: _propTypes.default.func
});

_defineProperty(CodeInput, "defaultProps", {
  onChange() {}

});